################################################################
# Copyright 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Lee Newberg
################################################################

# Authorization.tcl
# Routines for handling the "Authorization:" field in HTTP.

# Make Tcl recognize phxW3Authorization as an array.
set phxW3Authorization() ""
unset phxW3Authorization()

# phxW3AuthorizationGet
# Checks whether a user and passwd have been specified for the
# host (which is given by the url).  If so, it returns a command
# suitable for submission to the back end, phoenix.server.  If
# not, it returns the empty string.

proc phxW3AuthorizationGet url {
    global phxW3Authorization

    set host [phxW3AuthorizationURLtoHost $url]

    # Delete the pair if there is no user name set.
    if {[info exists phxW3Authorization($host)] &&
	[lindex $phxW3Authorization($host) 0] == {}} \
    {
	unset phxW3Authorization($host)
    }

    if {[info exists phxW3Authorization($host)]} {
	return [concat "HtAuthorization Basic" $phxW3Authorization($host)]
    } else {
	return ""
    }
}

proc phxW3AuthorizationInvent {} {
    global envlogname
    return [list HtAuthorization Basic $envlogname {}]
}

# phxW3AuthorizationSet
# 
# Requests (from the user) a username and password to associate
# with the host (which is given by the url).  The value is
# returned as a command which is suitable for submission to the
# back end, phoenix.server.  The value is also stored for
# retrieval by phxW3AuthorizationGet.  If the user cancels the
# operation then both the stored value and returned value are
# the empty string.

proc phxW3AuthorizationSet url {
    set host [phxW3AuthorizationURLtoHost $url]
    phxW3AuthorizationShow $host

    # Wait until the dialog box dies
    tkW3OutputCursorNormal
    tkwait window .auth
    tkW3OutputCursorWait

    return [phxW3AuthorizationGet $url]
}

# phxW3AuthorizationShow
#
# Create, if necesary, and show dialog box for Authorization.

proc phxW3AuthorizationShow host {
    toplevel .auth -height 225 -width 400
    wm transient .auth .
    wm title .auth "Edit Authorization"
    wm geometry .auth 500x200+70+180
    wm maxsize .auth 640 480

    frame .auth.data -relief flat
    frame .auth.buttonframe -relief flat

    place .auth.data -rely 0 -relheight .85 -relwidth 1
    place .auth.buttonframe -rely .85 -relheight .15 -relwidth 1

    label .auth.data.text -fg blue \
	-text "Insufficient Authorization for $host"
    place .auth.data.text -in .auth.data -rely 0 -relwidth .95

    label .auth.data.userlabel -text "User name"
    place .auth.data.userlabel -in .auth.data -rely .2

    entry .auth.data.user -relief sunken
    bind .auth.data.user <Return> "phxW3SelectionSetFocus .auth.data.passwd"
    bind .auth.data.user <Tab> "phxW3SelectionSetFocus .auth.data.passwd"
    place .auth.data.user -rely .4 -relx .025 -relwidth .95

    label .auth.data.passwdlabel -text "Password"
    place .auth.data.passwdlabel -rely .6

    entry .auth.data.passwd -relief sunken
    .auth.data.passwd config -fg [lindex [.auth.data.passwd config -bg] 4]
    .auth.data.passwd config -selectforeground \
	[lindex [.auth.data.passwd config -selectbackground] 4]
    bind .auth.data.passwd <Return> ".auth.buttonframe.ok invoke"
    bind .auth.data.passwd <Tab> "phxW3SelectionSetFocus .auth.data.user"
    place .auth.data.passwd -rely .80 -relx .025 -relwidth .95

    button .auth.buttonframe.ok -text OK \
	-command "phxW3AuthorizationOK $host"
    place .auth.buttonframe.ok -in .auth.buttonframe -relx .2 

    button .auth.buttonframe.cancel -text Cancel \
	-command "phxW3AuthorizationCancel $host"
    place .auth.buttonframe.cancel -in .auth.buttonframe -relx .75

    phxW3SelectionSetFocus .auth.data.user
}

proc phxW3AuthorizationOK host {
    global phxW3Authorization

    set phxW3Authorization($host) [list \
	[.auth.data.user get] [.auth.data.passwd get]]	
    destroy .auth
}

proc phxW3AuthorizationCancel host {
    .auth.data.user delete 0 end
    .auth.data.passwd delete 0 end
    phxW3AuthorizationOK $host
}

proc phxW3AuthorizationReauthorize {} {
    global phxW3Authorization

    set url [tkW3OutputGetAddress]
    catch {unset phxW3Authorization([phxW3AuthorizationURLtoHost $url])}
    tkW3NavigateReload
}

proc phxW3AuthorizationURLtoHost {url} {
    set x {^([^:/#?]+:)?(//([^:/#?]+)(:[0-9]+)?)?([^:#?]+)?(#[^/#?]+)?(\?[^:/#?]+)?$}
    regexp $x $url url method hostport host port path anchor query
    return $host
}