################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

################################################################
# forms.tcl contains routines relating to forms.  Any function
# which uses the glboal array HtForms should be in this file.
################################################################

################################################################
# Routines for submitting and resetting forms.
#

# phxW3FormsSubmit submits the form $form with given addtional
# name=value pairs

proc phxW3FormsSubmit {form args} {

    global HtForms

    # Build value string to append to action
    set valuestr [phxW3FormsBuildValues $form $args]

    # Set the method to POST if necessary.
    if {$HtForms($form,method) == "POST"} {phxW3NavigateMethodHTTP POST}

    # Goto the URL
    phxW3ModeRecordAndGoto "$HtForms($form,action)?$valuestr"

    # Restore the method to GET if we changed it.
    if {$HtForms($form,method) == "POST"} {phxW3NavigateMethodHTTP GET}
}

# This is called when the reset button in a form is hit

proc phxW3FormsReset {form} {

    global HtForms

    # For each form element
    for {set i 1} {$i <= $HtForms($form,numelems)} {incr i} {
	set elem "$form,elem=$i"

	# Get pertinent values for the element.
	set value [expr {[info exists HtForms($elem,value)] \
	    ? $HtForms($elem,value) : ""}]
	set checked [expr {[info exists HtForms($elem,checked)] \
	    ? $HtForms($elem,checked) : ""}]

	# Reset the element to the given default value(s).
	phxW3FormsSetValues $elem $value $checked
    }
}

# Called by FormsSubmit to build the value string

proc phxW3FormsBuildValues {form additions} {

    global HtForms
    set valstr ""

    # For each element in the form, get its name=value pair.
    for {set i 1} {$i <= $HtForms($form,numelems)} {incr i} {

	# Get value for current form element
	set elem "$form,elem=$i"
	set name [phxW3NavigateEncodeURL $HtForms($elem,name)]
	set values [phxW3FormsGetValues $elem]

	foreach value $values {
	    # Append name=value to value string
	    append valstr $name "=" [phxW3NavigateEncodeURL $value] "&"
	}
    }

    # Append any additions that were requested.
    foreach addition $additions {
	append valstr [phxW3NavigateEncodeURL [lindex $addition 0]] "=" \
	     [phxW3NavigateEncodeURL [lindex $addition 1]] "&"
    }

    return [string trim $valstr "&"]
}

# This is called by FormsBuildValue to get a value from the given widget.
# The return value is a list.  If the first element is zero then there
# is no "name=value" pair associated with the current state of the widget.
# If the first element is not zero then there is a second element.  It
# gives the value of "value".
# This mimics phxW3FormsSetValues to some degree!

proc phxW3FormsGetValues {elem} {
    global HtForms

    # Initially, assume that no values will be returned.
    set values {}

    # Note:
    # "reset" never returns a value.  "submit" and "formsimage"
    # only return values if they are invoked directly.  In such
    # a case, the name=value pair(s) are passed as the
    # "additions" argument to phxW3FormsBuildValues.

    switch $HtForms($elem,type) {
	password -
	text {
	    lappend values [$HtForms($elem,widget) get 1.0 end]
	}
	checkbox {
	    # Return a value only if this checkbutton has been selected.
	    if {$HtForms($elem,variable)} {
		lappend values $HtForms($elem,value)
	    }
	}
	radio {
	    set form $HtForms($elem,form)
	    set name $HtForms($elem,name)

	    # Return a value only if this radiobutton has been selected.
	    if {$HtForms($form,name=$name,variable) == $elem} {
		lappend values $HtForms($elem,value)
	    }
	}
	hidden {
	    lappend values $HtForms($elem,value)
	}
	select {
	    if {$HtForms($elem,multiple)} {
		for {set opt 0} \
		    {$opt < $HtForms($elem,numoptions)} \
		    {incr opt} \
		{
		    # Return a value only if this checkbutton is selected.
		    set option $elem,option=[expr {$opt + 1}]
		    if {$HtForms($option,variable)} {
			lappend values [expr {$HtForms($option,value)=={} \
			    ? [lindex [$HtForms($elem,menu) entryconfigure \
				$opt -label] 4] \
			    : [lindex $HtForms($option,value) 0]}]
		    }
		}
	    } else {
		set opt $HtForms($elem,variable)
		set option $elem,option=$opt
		lappend values [expr {$HtForms($option,value)=={} \
		    ? [lindex [$HtForms($elem,menu) entryconfigure \
			[expr {$opt-1}] -label] 4] \
		    : [lindex $HtForms($option,value) 0]}]
	    }
	}
	textarea {
		lappend values [$HtForms($elem,textarea) get 1.0 end]
	}
    }

    return $values
}

# This is called when a widget needs to be set to a certain
# value, eps. by FormsReset.  This mimics phxW3FormsGetValues to
# some degree!  Note that this does not change the default
# value, just the current value.

proc phxW3FormsSetValues {elem value checked} {
    global HtForms

    # Note:
    # "reset" and "formsimage" never have values.

    switch $HtForms($elem,type) {
	submit {
	    set form $HtForms($elem,form)
	    set name $HtForms($elem,name)
	    if {$name != ""} {
		$HtForms($elem,widget) configure -text $value -command \
		    [list phxW3FormsSubmit $form [list $name $value]]
	    } else {
		$HtForms($elem,widget) configure -text $value -command \
		    [list phxW3FormsSubmit $form]
	    }
	}
	password -
	text {
	    $HtForms($elem,widget) delete 1.0 end
	    $HtForms($elem,widget) insert 1.0 $value
	}
	checkbox {
	    set HtForms($elem,variable) $checked
	}
	radio {
	    set form $HtForms($elem,form)
	    set name $HtForms($elem,name)

	    # $HtForms($form,name=$name) should equal $elem if and only if
	    # $checked is true.
	    if {$checked} {
		set HtForms($form,name=$name) $elem
	    } elseif {$HtForms($form,name=$name) == $elem} {
		set HtForms($form,name=$name) ""
	    }
	}
	hidden {
	    # Nothing to do -- The only value is the default value.
	}
	select {
	    # "checked" is a list of the OPTIONs "selected" values.
	    if {$HtForms($elem,multiple)} {
		set opt 0
		foreach selected $checked {
		    set option $elem,option=[incr opt]
		    set HtForms($option,variable) $selected
		}
		$HtForms($elem,widget) configure -text \
		    $HtForms(SelectMultipleString)
	    } else {
		# Assume first that the first button is selected.
		set HtForms($elem,variable) 1

		# Check that assumption
		set opt 0
		foreach selected $checked {
		    incr opt
		    if {$selected} {
			set HtForms($elem,variable) $opt
			$HtForms($elem,menu) invoke [expr {$opt - 1}]
		    }
		}
	    }
	}
	textarea {
	    $HtForms($elem,textarea) delete 1.0 end
	    $HtForms($elem,textarea) insert 1.0 $HtForms($elem,value)
	}
    }
}

################################################################
# <FORM> and </FORM>:
################################################################

# phxW3FormsBeginForm is called when the <FORM> markup is
# encountered.  Each argument is a list.  If it is an empty list
# then no value was supplied for that parameter.  Note that a
# list with one element which is the empty string is not the
# same as an empty list!  Nonetheless, when we wish them to be
# equivalent, we'll take advantage of the fact that [lindex {{}} 0]
# is the same as [lindex {} 0].

proc phxW3FormsBeginForm {action enctype method} {

    global HtForms

    # If a previous form did not end, then end it now.
    if {$HtForms(in_form)} { phxW3FormsEndForm }

    # Set vars to store data of new form
    set HtForms(in_form) 1
    set form form=[incr HtForms(numforms)]

    # Action URL defaults to current page if not given
    set HtForms($form,action) \
	[expr {$action == {} ? [tkW3OutputGetAddress] : [lindex $action 0]}]

    # Method defaults to "GET"
    set HtForms($form,method) \
	[expr {$method == {} ? "GET" : [lindex $method 0]}]

    # Enctype defaults to "application/x-www-form-urlenced" if the 
    # method is "POST", otherwise it defaults to "".
    set HtForms($form,enctype) \
	[expr {$HtForms($form,method) == "POST" && $enctype == {} ? \
	    "application/x-www-form-urlencoded" : [lindex $enctype 0]}]

    # Clear count of widgets
    set HtForms($form,numelems) 0
}

# phxW3FormsEndForm is called when the </FORM> markup is
# encountered.

proc phxW3FormsEndForm {} {

    global HtForms

    # Do nothing if we're not already building a form.
    if {$HtForms(in_form) == 0} return

    if {$HtForms(in_textarea)} { phxW3FormsEndTextarea } \
    elseif {$HtForms(in_select)} { phxW3FormsEndSelect }

    # We're not building a form anymore
    set HtForms(in_form) 0
}

################################################################
# <INPUT>:  (There is no </INPUT>)
################################################################

# phxW3FormsInsertInput is called when the <INPUT> markup is
# encountered.  Each argument is a list -- see the comments for
# phxW3FormsBeginForm.

proc phxW3FormsInsertInput \
    {align checked maxlength name size src type value} \
{
    global HtForms

    # If we're not already building a form then give up.
    if {$HtForms(in_form)==0 || \
	($name=={} && $type!="reset" && $type!="submit")} return

    # If a previous select or textarea did not end, then end it now.
    if {$HtForms(in_textarea)} { phxW3FormsEndTextarea } \
    elseif {$HtForms(in_select)} { phxW3FormsEndSelect }

    # Elements such as input, textarea, and select are stored in
    # HtForms array with indexes composed of text, and the form
    # and element numbers.
    set form form=$HtForms(numforms)
    set elem $form,elem=[incr HtForms($form,numelems)]

    # The arguments are lists.  Extract the first element of each list,
    # using the default value if the list is empty.
    set type [expr {$type=={} ? "text" : [lindex $type 0]}]

    if {$align=={}} { set align {BOTTOM} }
    # Make sure ALIGN option is legal!!!

    set checked [phxW3FormsBoolean $checked]

    set maxlength [lindex $maxlength 0]
    if {![regexp {^[0-9]+$} $maxlength]} { set maxlength "unlimited" }

    set name [lindex $name 0]

    set size [lindex $size 0]
    if {![regexp {^[0-9]+(,[0-9]+)*$} $size]} {
	switch $type {
	    text -
	    password { set size 24 }
	    default { set size undefined }
	}
    }

    set src [lindex $src 0]

    if {$value=={}} {
	if {$type=="submit"} { set value "Submit" } \
	elseif {$type=="reset"} { set value "Reset" }
    } else { set value [lindex $value 0] }

    # Save the arguments associated with this widget.
    set HtForms($elem,elem) "input"
    set HtForms($elem,form) $form
    set HtForms($elem,align) $align
    set HtForms($elem,checked) $checked
    set HtForms($elem,maxlength) $maxlength
    set HtForms($elem,name) $name
    set HtForms($elem,type) $type
    set HtForms($elem,size) $size
    set HtForms($elem,src) $src
    set HtForms($elem,value) $value

    # Build the new widget according to type
    set widget {}

    # The for loop is a hack that allows the "break" commands to break
    # from the switch statement.
    for {set i 1} {$i <= 1} {incr i} {
    switch $type {
    checkbox {
	set widget [phxW3WidgetInsertFormsCheckbox $checked $name $size $value]
	if {$widget == ""} { break }
	set HtForms($elem,variable) $checked
	$widget configure -variable HtForms($elem,variable)
    }
    radio {
	set widget [phxW3WidgetInsertFormsRadio $checked $name $size $value]
	if {$widget == ""} { break }
	$widget configure -variable HtForms($form,name=$name,variable) \
	    -value $elem
	if {$checked} {$widget select} else {$widget deselect}
    }
    hidden {
	# Set widget name to be the empty string.
	set widget [list {}]
	# Nothing else to do.  The ability to insert and delete hidden fields
	# in edit mode may change this!!
    }
    image {
	# This code mimics tkW3HtSetImg to some degree!!

	# A single "src" must be supplied.
	if {[llength $src] != 1} { break }

	# Convert the src URL to absolute addressing
	set address [tkW3OutputGetAddress]
	set src [tkW3NavigateServerSend [list HtParseName $src $address]]

	# Get a local copy of the bitmap
	set cmd [tkW3NavigateServerSend [list HtLoad $src]]
	if {[regexp {(^Error)} $cmd]} { break }

	# Extract the name of the local copy.
	set local ""
	if {[llength $cmd] == 6 && [lindex $cmd 2] != "tkW3HtSetName"} {
	    set local [[lindex $cmd 0] [lindex $cmd 1] [lindex $cmd 2]]
	}
	if {$local == ""} { break }

	# Create the widget for this image.
	set widget [phxW3WidgetInsertFormsImage $align $name $size $local]
	if {$widget == ""} { break }

	# Set the binding for the widget
	bind $widget <ButtonRelease-1> \
	    "phxW3ModeSingleClickFormsImage %W $form $name %x %y"
    }
    password {
	set widget \
	    [phxW3WidgetInsertFormsPassword $maxlength $name $size $value]
    }
    reset {
	set widget [phxW3WidgetInsertFormsReset $size $value]
	if {$widget == ""} { break }
	$widget configure -command "phxW3FormsReset $form"
    }
    submit {
	set widget [phxW3WidgetInsertFormsSubmit $name $size $value]
	if {$widget == ""} { break }
	# We contribute a name=value pair if a name is defined.
	if {$name != ""} {
	    $widget configure -command \
		[list phxW3FormsSubmit $form [list $name $value]]
	} else {
	    $widget configure -command [list phxW3FormsSubmit $form]
	}
    }
    text {
	set widget [phxW3WidgetInsertFormsText $maxlength $name $size $value]
    }
    }
    }

    if {$widget != ""} {
	set HtForms($elem,widget) $widget
	if {$widget != [list ""]} { set HtForms(widget=$widget) $elem }
    } else {
	# Abandon this input element
	incr HtForms($form,numelems) -1
    }
}

################################################################
# <SELECT>, <OPTION>, and </SELECT>: (They have a syntactic
# structure similar to <UL>, <LI>, and </UL>.)
################################################################

# phxW3FormsBeginSelect is called when the <SELECT> markup is
# encountered.  Each argument is a list -- see the comments for
# phxW3FormsBeginForm.

proc phxW3FormsBeginSelect {multiple name} {
    global HtForms

    # If we're not already building a form then give up.
    if {$HtForms(in_form) == 0 || $name == {}} return

    # If a previous select or textarea did not end, then end it now.
    if {$HtForms(in_textarea)} { phxW3FormsEndTextarea } \
    elseif {$HtForms(in_select)} { phxW3FormsEndSelect }

    set HtForms(in_select) 1

    # Elements such as input, textarea, and select are stored in
    # HtForms array with indexes composed of text, and the form
    # and element numbers.
    set form form=$HtForms(numforms)
    set elem $form,elem=[incr HtForms($form,numelems)]

    # The arguments are lists.  Extract the first element of each list,
    # using the default value if the list is empty.
    set name [lindex $name 0]
    set multiple [phxW3FormsBoolean $multiple]

    # Save the arguments associated with this widget.
    set HtForms($elem,elem) "select"
    set HtForms($elem,type) "select"
    set HtForms($elem,form) $form
    set HtForms($elem,name) $name
    set HtForms($elem,multiple) $multiple

    # Initialize the widget
    set HtForms($elem,numoptions) 0
    set HtForms($elem,widget) \
	[phxW3WidgetInsertFormsSelect $name $multiple]
    set HtForms(widget=$HtForms($elem,widget)) $elem
    set HtForms($elem,menu) [lindex [$HtForms($elem,widget) configure -menu] 4]

    # Until we know otherwise, assume that the first button has been selected.
    if {$multiple == 0} { set HtForms($elem,variable) 1 }
}

# phxW3FormsEndSelect is called when the </SELECT> markup is
# encountered.  

proc phxW3FormsEndSelect {} {
    global HtForms

    # Do nothing if we're not already building a Select.
    if {$HtForms(in_select) == 0} return

    set HtForms(in_select) 0
    set form form=$HtForms(numforms)
    set elem $form,elem=$HtForms($form,numelems)
    set widget $HtForms($elem,widget)
    set menu $HtForms($elem,menu)

    # Put some text on the menubutton for this SELECT.
    if {$HtForms($elem,multiple)} {
	$widget configure -text $HtForms(SelectMultipleString) -relief sunken
    } else {
	# Compute and set the size of the menubutton label.  Doesn't work!!!
	set max 0
	for {set i [expr {$HtForms($elem,numoptions)-1}]} {$i>=0} {incr i -1} {
	    $menu invoke $i
	    set length [lindex [$widget configure -width] 4]
	    if {$length > $max} {set max $length}
	}
	$widget configure -width $max

	# Display something as the menu button label
	$menu invoke [expr {$HtForms($elem,variable)-1}]
    }

    # Remember the set of "checked" values for the OPTIONs.
    if {[info exists HtForms($elem,value)]} { unset HtForms($elem,value) }
    set HtForms($elem,checked) ""
    for {set opt 1} {$opt <= $HtForms($elem,numoptions)} {incr opt} {
	lappend HtForms($elem,checked) $HtForms($elem,option=$opt,selected)
    }
}

# phxW3FormsAddSelect is called when plain text is encountered
# within a <SELECT> ... </SELECT> pair.

proc phxW3FormsAddSelect {string} {
    global HtForms

    set form form=$HtForms(numforms)
    set elem $form,elem=$HtForms($form,numelems)
    set option $HtForms($elem,numoptions)
    set widget $HtForms($elem,widget)
    set menu $HtForms($elem,menu)

    # Ignore text before the first <OPTION> markup.
    if {$option == 0} return
    set menuSize [$menu index last]
    if {$menuSize == "none"} { set menuSize 0 }

    if {$option == $menuSize + 1} {
	# We're appending to current text
	append current [lindex [$menu entryconfigure last -label] 4]
	$menu delete last
    }
    append current $string
    if {$HtForms($elem,multiple)} {
	# Add a checkbutton entry
	$menu add checkbutton -label $current \
	    -variable HtForms($elem,option=$option,variable)
	set HtForms($elem,option=$option,variable) \
	    $HtForms($elem,option=$option,selected)
    } else {
	# Add a radiobutton entry
	$menu add radiobutton -label $current \
	    -variable HtForms($elem,variable) -value $option \
	    -command "phxW3FormsSelectRadioCommand $elem"
    }
}

# phxW3FormsSelectRadioCommand changes the text displayed on the
# menu button.

proc phxW3FormsSelectRadioCommand {elem} {
    global HtForms

    # Menu entries are a zero-based array.  Options are one-based.
    set menuentry [expr {$HtForms($elem,variable) - 1}]

    $HtForms($elem,widget) configure -text \
"\[X\] [lindex [$HtForms($elem,menu) entryconfigure $menuentry -label] 4]"

}

# phxW3FormsInsertOption is called when the <OPTION> markup is
# encountered.  Each argument is a list -- see the comments for
# phxW3FormsBeginForm.

proc phxW3FormsInsertOption {selected value} {
    global HtForms

    # If we're not already building a Select then give up.
    if {$HtForms(in_select) == 0} return

    # The arguments are lists.  Extract the first element of each list,
    # using the default value if the list is empty.
    set selected [phxW3FormsBoolean $selected]
    # "value" must remain as a list so that phxW3FormsGetValues can work!

    set form form=$HtForms(numforms)
    set elem $form,elem=$HtForms($form,numelems)
    set option $elem,option=[incr HtForms($elem,numoptions)]
    set HtForms($option,selected) $selected
    set HtForms($option,value) $value

    # Call phxW3AddSelect so that an (initially empty) OPTION will be
    # created.
    phxW3FormsAddSelect ""
}

################################################################
# <TEXTAREA> and </TEXTAREA>:
################################################################

# phxW3FormsBeginTextarea is called when the <TEXTAREA> markup
# is encountered.  Each argument is a list -- see the comments
# for phxW3FormsBeginForm.

proc phxW3FormsBeginTextarea {cols name rows} {
    global HtForms

    # If we're not already building a form then give up.
    if {$HtForms(in_form) == 0 || $name == {}} return

    # If a previous select or textarea did not end, then end it now.
    if {$HtForms(in_textarea)} { phxW3FormsEndTextarea } \
    elseif {$HtForms(in_select)} { phxW3FormsEndSelect }

    set HtForms(in_textarea) 1

    # Elements such as input, textarea, and select are stored in
    # HtForms array with indexes composed of text, and the form
    # and element numbers.
    set form form=$HtForms(numforms)
    set elem $form,elem=[incr HtForms($form,numelems)]

    # The arguments are lists.  Extract the first element of each list,
    # using the default value if the list is empty.
    set name [lindex $name 0]

    set rows [lindex $rows 0]
    if {![regexp {^[0-9]+$} $rows]} { set rows 24 }

    set cols [lindex $cols 0]
    if {![regexp {^[0-9]+$} $cols]} { set cols 80 }

    # Set the values passed for the current element
    set HtForms($elem,elem) "textarea"
    set HtForms($elem,type) "textarea"
    set HtForms($elem,form) $form
    set HtForms($elem,name) $name
    set HtForms($elem,rows) $rows
    set HtForms($elem,cols) $cols

    # Initialize the widget
    set HtForms($elem,widget) \
	[phxW3WidgetInsertFormsTextarea $name $rows $cols]
    set HtForms(widget=$HtForms($elem,widget)) $elem
    set HtForms($elem,vscroll) [lindex [pack slaves $HtForms($elem,widget)] 0]
    set HtForms($elem,textarea) [lindex [pack slaves $HtForms($elem,widget)] 1]

    # The textarea is empty so far.
    set HtForms($elem,value) ""
}

# phxW3FormsEndTextarea is called when the </TEXTAREA> markup
# is encountered.

proc phxW3FormsEndTextarea {} {
    global HtForms

    # Do nothing if we're not already building a Textarea.
    if {$HtForms(in_textarea) == 0} return

    set HtForms(in_textarea) 0
}

# phxW3FormsAddTextarea is called when plain text is encountered
# within a <TEXTAREA> ... </TEXTAREA> pair.

proc phxW3FormsAddTextarea {string} {
    global HtForms

    set form form=$HtForms(numforms)
    set elem $form,elem=$HtForms($form,numelems)
    set textarea $HtForms($elem,textarea)
    $textarea insert end $string
    set HtForms($elem,value) [$textarea get 1.0 end]
}

# phxW3FormsBoolean returns 0 if its argument is an empty list or a
# list containing a synonym of "NO".  Otherwise it returns 1.

proc phxW3FormsBoolean {value} {
    set bool 1
    set value [string toupper $value]
    foreach no {{} {0} {N} {NO} {OFF}} {
	if {$value == $no} {set bool 0 ; break}
    }
    return $bool
}



################################################################
# Routines for converting Forms widgets to HTML
#

# phxW3FormsConvertFormToHTML:
# 
# This routine outputs markup for the forms element encountered.
# If there are <INPUT TYPE="hidden"> elements between this
# element and the next visible element then it outputs markup
# for them, too.  If this is the first visible element then it
# outputs the <FORM> markup and any previous hidden elements,
# too.  If this is the last visible element, then it outputs
# </FORM> markup, too.

# Note that any form with only hidden elements will not have its
# HTML generated!!

proc phxW3FormsConvertFormToHTML {widget} {
    global HtForms

    set elem $HtForms(widget=$widget)
    regexp {^form=[0-9]+,elem=([0-9]+)$} $elem {} elnum
    set form $HtForms($elem,form)
    
    # Check if this is the first visible element.
    for {set e [expr $elnum-1]} \
	{$e >= 1 && $HtForms($form,elem=$e,type) == "hidden"} \
	{incr e -1} \
	{}
    if {$e == 0} {
	# We've got the first visible element.  Output <FORM> markup
	# and any hidden elements before the current element.
	append ret_val "<FORM"
	append ret_val " ACTION=\"$HtForms($form,action)\""
	append ret_val " METHOD=\"$HtForms($form,method)\">"
	for {set e 1} {$e < $elnum} {incr e} {
	    append ret_val [phxW3FormsConvertElementToHTML $form,elem=$e]
	}
    }

    # Output the current element.
    append ret_val [phxW3FormsConvertElementToHTML $elem]

    # Output any hidden elements between this element and the
    # next visible element.  If there is no next visible
    # element, output </FORM> markup.

    for {set e [expr $elnum+1]} \
	{$e <= $HtForms($form,numelems) && \
	    $HtForms($form,elem=$e,type) == "hidden"} \
	{incr e} \
	{append ret_val [phxW3FormsConvertElementToHTML $form,elem=$e]}

    if {$e > $HtForms($form,numelems)} {append ret_val "</FORM>"}

    return $ret_val
}

# phxW3FormsConvertElementToHTML returns a string of HTML for a
# given forms element.

proc phxW3FormsConvertElementToHTML {elem} {
    global HtForms

    set widget $HtForms($elem,widget)
    switch $HtForms($elem,elem) {
	input {
	    append html "<INPUT TYPE=$HtForms($elem,type)"
	    switch $HtForms($elem,type) {
		checkbox {
		    append html " NAME=\"$HtForms($elem,name)\""
		    append html " VALUE=\"$HtForms($elem,value)\""
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    if {$HtForms($elem,variable)} { append html " CHECKED" }
		}
		radio {
		    append html " NAME=\"$HtForms($elem,name)\""
		    append html " VALUE=\"$HtForms($elem,value)\""
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    set form $HtForms($elem,form)
		    set name $HtForms($elem,name)
		    if {$HtForms($form,name=$name,variable) == $elem} {
			append html " CHECKED"
		    }
		}
		hidden {
		    append html " NAME=\"$HtForms($elem,name)\""
		    append html " VALUE=\"$HtForms($elem,value)\""
		}
		image {
		    append html " NAME=\"$HtForms($elem,name)\""
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    append html " ALIGN=\"$HtForms($elem,align)\""
		    append html " SRC=\"$HtForms($elem,src)\""
		}
		password -
		text {
		    append html " NAME=\"$HtForms($elem,name)\""
		    append html " VALUE=\"$HtForms($elem,value)\""
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    if {$HtForms($elem,maxlength) != "unlimited"} {
			append html " MAXLENGTH=\"$HtForms($elem,maxlength)\""
		    }
		}
		reset {
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    if {$HtForms($elem,value) != "Reset"} {
			append html " VALUE=\"$HtForms($elem,value)\""
		    }
		}
		submit {
		    if {$HtForms($elem,name) != ""} {
			append html " NAME=\"$HtForms($elem,name)\""
		    }
		    if {$HtForms($elem,size) != "undefined"} {
			append html " SIZE=\"$HtForms($elem,size)\""
		    }
		    if {$HtForms($elem,value) != "Submit"} {
			append html " VALUE=\"$HtForms($elem,value)\""
		    }
		}
	    }
	    append html ">"
	}
	select {
	    append html "<SELECT NAME=\"$HtForms($elem,name)\""
	    if {$HtForms($elem,multiple)} { append html " MULTIPLE" }
	    append html ">"
	    for {set opt 1} {$opt <= $HtForms($elem,numoptions)} {incr opt} {
		append html "<OPTION"
		set option $elem,option=$opt
		if {$HtForms($elem,multiple)} {
		    if {$HtForms($option,variable)} {
			append html " SELECTED"
		    }
		} else {
		    if {$HtForms($elem,variable) == $opt} {
			append html " SELECTED"
		    }
		}
		if {$HtForms($option,value) != {}} {
		    append html " VALUE=\"[lindex $HtForms($option,value) 0]\""
		}	    
		append html ">" [lindex [$HtForms($elem,menu) entryconfigure \
		    [expr {$opt-1}] -label] 4]
	    }
	    append html "</SELECT>"
	}
	textarea {
	    append html "<TEXTAREA"
	    append html " NAME=\"$HtForms($elem,name)\""
	    append html " COLS=\"$HtForms($elem,cols)\""
	    append html " ROWS=\"$HtForms($elem,rows)\""
	    append html ">" [$HtForms($elem,textarea) get 1.0 end]
	    append html "</TEXTAREA>"
	}
    }
    return $html
}

################################################################
# phxW3FormsInitialize is called when Phoenix starts.

proc phxW3FormsInitialize {} {
    global HtForms

    catch {unset HtForms}
    set HtForms(numforms) 0
    set HtForms(in_form) 0
    set HtForms(in_textarea) 0
    set HtForms(in_select) 0
    set HtForms(SelectMultipleString) {[X] Choose a few...}
    set HtForms(output_form) "none"
}
