################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
# Tim Pierce
################################################################

# USAGE:	DLG:<type> parent name message button_1 button_2 ...
#
# parent	The name of the window to parent from
# name		The name to use for the toplevel of this dialog
# message	Message to print in the message area.
# button_{1,2,3} Text to put on each button.
#
#
#
# This starts a dialog of the type indicated in <type>.  It has [1..3]
# buttons, depending on how many (non-null) strings are passed.  It does
# a passive grab, to force the user to respond.  It waits until one of
# the buttons are pressed, then kills off the dialog and returns.  The
# return value is the number for the button which was pressed {1,2,3}.
#
# THIS FILE REQUIRES tk2.2 or greater.

set DLGBitmapPath $tkW3SourcePath/bitmap

proc DLG:information {parent w msg args} {
    global DLGBitmapPath
    DLG:build "$parent" "$w" "$msg" @$DLGBitmapPath/information "$args"
}

proc DLG:warning {parent w msg args} {
    global DLGBitmapPath
    DLG:build "$parent" "$w" "$msg" @$DLGBitmapPath/warning "$args"
}

proc DLG:error {parent w msg args} {
    global DLGBitmapPath
    DLG:build "$parent" "$w" "$msg" @$DLGBitmapPath/error "$args"
}

proc DLG:question {parent w msg args} {
    global DLGBitmapPath
    DLG:build "$parent" "$w" "$msg" @$DLGBitmapPath/question "$args"
}

proc DLG:working {parent w msg args} {
    global DLGBitmapPath
    DLG:build "$parent" "$w" "$msg" @$DLGBitmapPath/working "$args"
}

# Create an entry dialog
# Calling arguments are
# parent - dialog to be centered on
# w - name of dialog
# message - a list of entry labels
# command_output - what to put in the command button
# command - command to execute if command button is pressed

proc DLG:entry {parent w title message command_output command} {
    DLG:toplevel $parent $w $title
    DLG:draw_entries $w $message
    DLG:draw_buttons $w [list "$command_output" "Cancel"]

    DLG:bind_entry $w [llength $message] "DLG:invoke_button $w 1"
    DLG:bind_button $w 1 "DLG:hide $w
eval $command \[DLG:get_entry_values $w [llength $message]\]"
    DLG:bind_button $w 2 "DLG:hide $w"
    return $w
}

proc DLG:listbox {parent w title list_items command_output command} {
    DLG:toplevel $parent $w
    DLG:draw_listbox $w $list_items
    DLG:draw_buttons $w [list "$command_output" "Cancel" "Help"]

    DLG:bind_button $w 1 "DLG:hide $w
eval $command \[DLG:get_listbox_highlighted $w\]"
    DLG:bind_button $w 2 "DLG:hide $w"
    DLG:bind_button $w 3 "tkWWWHelpNoHelp"
    return $w
}

# The next procedures manage the dialog boxes

proc DLG:show {parent dialog {minwidth 0} {minheight 0}} {
    # First update to make sure the boxes are the right size
    update idletask

    # If window already mapped, then just bring it to foreground -twp
    if {[wm state $dialog] == "normal"} {
	wm withdraw $dialog
    } else {
	# Set the position and update
	DLG:position $parent $dialog $minwidth $minheight
	update idletask
    }

    # and now make it visible. Viola!  Centered over parent.
    wm deiconify $dialog

    # Make sure we set the focus.
    if {[winfo exists $dialog.ef1]} {
	phxW3SelectionSetFocus $dialog.ef1.entry_frame.entry
    }
}

proc DLG:position {parent dialog minwidth minheight} {
    # Tell the WM that we'll do this ourselves.
    wm sizefrom $dialog user
    wm positionfrom $dialog user

    # Where is my parent and what are it's dimensions
    set pargeo [split [wm geometry $parent] "+x"]
    set parwidth [lindex $pargeo 0]
    set parheight [lindex $pargeo 1]
    set parx [lindex $pargeo 2]
    set pary [lindex $pargeo 3]

    # What are my dimensions ?
    set dialogwidth [winfo reqwidth $dialog]
    set dialogheight  [winfo reqheight $dialog]

    if {$dialogwidth < $minwidth} {
      set dialogwidth $minwidth
    }
    if {$dialogheight < $minheight} {
      set dialogheight $minheight
    }
    wm minsize $dialog $minwidth $minheight

    # What are is the offset of the virtual window
    set vrootx [winfo vrootx $parent]
    set vrooty [winfo vrooty $parent]

    set dialogx [expr $parx+($parwidth-$dialogwidth)/2+$vrootx]
    set dialogy [expr $pary+($parheight-$dialogheight)/2+$vrooty]

    set maxx [expr "[winfo screenwidth $parent] - $dialogwidth"]
    set maxy [expr "[winfo screenheight $parent] - $dialogheight"]

# Make sure it doesn't go off screen
    if {$dialogx < 0} {
	set dialogx 0
    } {
	if {$dialogx > $maxx} {
	    set dialogx $maxx
	}
    }
    if {$dialogy < 0} {
	set dialogy 0
    } {
	if {$dialogy > $maxy} {
	    set dialogy $maxy
	}
    }

    set dialogx [expr $dialogx-$vrootx]
    set dialogy [expr $dialogy-$vrooty]

    # Build my new position (and dimensions)
    wm geometry $dialog [ format "%dx%d+%d+%d" $dialogwidth $dialogheight \
			 $dialogx $dialogy]

}

proc DLG:hide {w} {
    # If the focus is on $w, return it to "focus default"
    if [string match $w.* [focus]] {
#	focus [focus default]
    }

    # If $w owns the selection, clear it.
    if {[phxW3SelectionGet] == $w} {
#	phxW3SelectionClear $w
    }

    # Withdraw the window.
    wm withdraw $w
}

proc DLG:destroy {w} {
    destroy $w
}

# These are procedures for building the dialog boxes
# You can use them to create new types of boxes

# Create a toplevel window with dialog box characteristics

proc DLG:toplevel {parent w {title ""}} {
    catch { destroy $w }    
    toplevel $w -class Dialog

    # Lets the dialog window be handled like Motif dialogs by the WM
    wm group $w $parent
    wm transient $w $parent
    wm minsize $w 0 0

    wm title $w $title
    wm withdraw $w

    return $w
}

# Draw a frame with an icon and a message
proc DLG:draw_icon_msg {w iconfile msg} {
    pack configure [frame $w.msg] -side top -fill both -expand yes

    pack configure [label $w.msg.icon -bitmap $iconfile] -side left
    pack configure \
	[message $w.msg.msg  -text "$msg" -justify left -aspect 1500] \
	-side left -fill both -expand yes
}

# Build a modal dialog box
proc DLG:build {parent w msg iconfile blist} {
    set done "DLG[set w]done"
    global $done
    set $done 0

    DLG:toplevel $parent $w
    DLG:draw_icon_msg $w $iconfile $msg
    DLG:draw_buttons $w $blist
    for {set i 1} {$i <= [llength $blist]} {incr i 1} {
	DLG:bind_button $w $i "global $done; set $done $i"
    }

    DLG:show $parent $w
    grab $parent
    tkwait variable $done
    grab release $parent
    DLG:destroy $w
    return "[set $done]"
}

# Draw some entries in your box
proc DLG:draw_entries {w llist} {
    set index 1
    pack configure [frame $w.top_frame -borderwidth 2 -relief flat] \
	-side top -fill x -expand yes
    foreach label $llist {
	set name ef$index
	pack configure [frame $w.$name -borderwidth 5 -relief flat] \
	    -in $w.top_frame -side top -fill x -expand yes

	# Add the entry widget
	pack configure [label $w.$name.label -text "$label"] \
	    -side left 
	pack configure [frame $w.$name.entry_frame -relief flat] \
	    -side right -fill x -expand yes

	# Must use an extra frame to accommodate scrollbar
	pack configure \
	    [entry $w.$name.entry_frame.entry \
		-relief sunken \
		-scrollcommand "$w.$name.entry_frame.sc set"] \
	    -side top -fill x -expand yes

	pack configure \
	    [scrollbar $w.$name.entry_frame.sc \
		-orient horiz \
		-width 10 \
		-borderwidth 1 \
		-activeforeground gray \
		-command "$w.$name.entry_frame.entry view"] \
	    -side bottom -fill x

	# Set entry bindings. (twp)

	phxW3SelectionWidgetInitialize $w.$name.entry_frame.entry

	incr index 1
    }
}

proc DLG:delete_selection {w} {
    if {[selection own] == $w} {
	$w delete sel.first sel.last
	selection clear $w
    }
}

# Add bindings to the entries
proc DLG:bind_entry {w index binding} {
    bind $w.ef$index.entry_frame.entry <Return> $binding
}

# Get values from entries
proc DLG:get_entry_values {w number} {
    set return_string ""
    for {set i 1} {$i <= $number} {incr i 1} {
	lappend return_string [$w.ef$i.entry_frame.entry get]
    }
    return $return_string
}

proc DLG:get_entry_value {w i} {
    $w.ef$i.entry_frame.entry get
}

# Set entry values
proc DLG:set_entry_value {w index message} {
    $w.ef$index.entry_frame.entry delete 0 end
    $w.ef$index.entry_frame.entry insert 0 $message
}

# create a listbox

proc DLG:draw_listbox {w elements} {
    pack configure [frame $w.frame -borderwidth 5 -relief flat] \
	-side top -fill both -expand yes
    pack configure [listbox $w.list -relief sunken ] \
	-in $w.frame -side top -fill both -expand yes
    foreach elem $elements {
	$w.list insert end $elem
    }
}

proc DLG:get_listbox_highlighted {w} {
    $w.list curselection
}

# Draw the buttons at the bottom
proc DLG:draw_buttons  { w blist } {
    pack configure [frame $w.cmds -relief flat] -side top -fill x

    # The extra frames below are to indicate current focus.  But we don't
    # have anything to handle keyboard focus.  Besides, the Motif
    # implementation uses a `knurled' look to the frame around buttons
    # in their standard dialogs and we must use two layers of frame to
    # implement that (see above).
    #
    # Also tried to set up stuff for keyboard focus, but it did a couple of
    # weird things and there isn't time to fix them now.  Keyboard focus
    # remained wherever the last `focus' command had gone, regardless of
    # where the pointer focus was.  This is a bug with the regular FOCUS
    # module as well.  The cure will be to watch Enter and Leave events
    # to know which window the pointer is in.  But this seems to be wrong
    # as well: What if the users configures their window manager for
    # a click-to-focus mode.  Won't we still get Enter&Leave events
    # regardless of where they've clicked-to-focus?

    set index 1
    foreach btn $blist {
	if {"$btn" != ""} {
	    pack configure \
		[ frame $w.cmds.f$index -borderwidth 2 -relief flat ] \
		-side left -fill both -expand yes
	    pack configure [label $w.cmds.f$index.left -text "" ] \
		-side left -expand yes
	    pack configure [label $w.cmds.f$index.right -text "" ] \
		-side right -expand yes
	    pack configure \
		[button $w.cmds.f$index.btn -text "$btn" -anchor center] \
		-side left -fill x

#	     DLG:keybindings $w.cmds.f$index.btn$index "[lindex $btn$index 1]"
	}
	incr index 1
    }
}    

proc DLG:bind_button {w index command} {
	if [catch {winfo class $w}] { return }
    $w.cmds.f$index.btn config -command $command
}

proc DLG:invoke_button {w index} {
    $w.cmds.f$index.btn invoke
}

# Future expansion: For when we are able to do keyboard focus and
# traversal in these things.
#proc DLG:keybindings { w cmd } {
#	bind $w <Key-space> "$cmd"
#	bind $w <Key-Return> "$cmd"
#}
