/* 
 * tclTest.c --
 *
 *	Test driver for TCL.
 *
 * Copyright 1987-1991 Regents of the University of California
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appears in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

/* This file has been heavily modified for use with the phoenix.server
 * program.
 */

#ifndef STDIO_H
#include <stdio.h>
#define STDIO_H
#endif
#include <errno.h>
#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
/* An ANSI string.h and pre-ANSI memory.h might conflict.  */
#if !STDC_HEADERS && HAVE_MEMORY_H
#include <memory.h>
#endif /* not STDC_HEADERS and HAVE_MEMORY_H */
#else /* not STDC_HEADERS and not HAVE_STRING_H */
#include <strings.h>
#define strchr index
#define strrchr rindex
 memcpy(d, s, n) bcopy ((s), (d), (n))
 memcmp(s1, s2, n) bcmp((s1), (s2), (n))
 memset(s, ignore, n) bzero((s), (n))
/* memory.h and strings.h conflict on some systems.  */
#endif /* not STDC_HEADERS and not HAVE_STRING_H */
#include <tcl.h>

/* Get prototypes */
#ifdef STDC_HEADERS
#include <stdlib.h>
#define _PROTO(x) x
#else
extern int exit();
#define _PROTO(x)  ()
#endif

/* Recognize init_tkWWW */
extern void init_tkWWW _PROTO((Tcl_Interp *));
int quitFlag = 0;

int main()
{
  char line[1000];
  int result, gotPartial;
  char *prompt = "phoenix:";

  /* Intialize interpreter and buffer */
  Tcl_Interp *interp = Tcl_CreateInterp();
  Tcl_DString buffer[1];
  Tcl_DStringInit(buffer);

  /* Add tkWWW commands */
  init_tkWWW(interp);

  /* Put out the header */
  printf("Phoenix Server Version %s\n", SERVER_VERSION);

  gotPartial = 0;
  while (1) {
    clearerr(stdin);
    if (!gotPartial) {
      puts(prompt);
      fflush(stdout);
    }
    if (fgets(line, 1000, stdin) == NULL) {
      if (!gotPartial) 
	exit(0);
      line[0] = '\0';
    }
    Tcl_DStringAppend(buffer, line, -1);
    if (!Tcl_CommandComplete(buffer->string)) {
      gotPartial = 1;
      continue;
    }

    gotPartial = 0;
    result = Tcl_RecordAndEval(interp, buffer->string, 0);
    if (result == TCL_OK) {
      if (*interp->result)
	puts(interp->result);

      if (quitFlag) {
	Tcl_DeleteInterp(interp);
	Tcl_DStringFree(buffer);
	exit(0);
      }
    } else {
      fputs("Error", stdout);
      if (result != TCL_ERROR) 
	printf(" %d", result);
	
      if (*interp->result) {
	fputs(": ", stdout);
	fputs(interp->result, stdout);
      }
      fputs("\n", stdout);
    }
    Tcl_DStringFree(buffer);
  }
}
